@@ -82,7 +82,21 @@ class UrlEnum(Enum):
                                       )
                        )
     ''' 查询指数行情收盘价格 '''
-    
+    MARKET_SPJG_BATCH = UrlCfg('http://%s/data/zs/spj/batch', 'post', 'ams',
+                        lambda x: {
+                           'sczs': x.get('sczs'),
+                           'gmclzs': x.get('gmclzs'),
+                           'smclzs': x.get('smclzs'),
+                           'startTime': x['startDate'],
+                           'endTime': x['endDate'],
+                           'fields': x['fields']
+                        },
+                        lambda y: {},
+                       supportFields=('hb1n', 'hb1y', 'hb1z', 'hb2n', "hb3n","hb3y","hb5n",
+                                      "hb6y","hbcl","hbjn","jjsl","jyrq","scdm","spjg","tjrq","zqdm"
+                                      )
+                       )
+    ''' 查询指数行情收盘价格多个指数代码 '''    
 
 ################################ 公募 ################################
     # 公募基金净值
@@ -100,6 +114,20 @@ class UrlEnum(Enum):
                        supportFields=('jzrq', 'jjdm', 'jjjz', 'ljjz', 'hbcl','hbdr','fqdwjz')
                        )
     '''公募基金净值'''
+    # 公募基金净值多基金代码
+
+    ALL_JJJZ = UrlCfg('http://%s/data/all/jz', 'post', 'ams',
+                       lambda x: {
+                            'gmdm': x.get('gmdm'),
+                            'smdm': x.get('smdm'),
+                            'startTime': x['startDate'],
+                            'endTime': x['endDate'],
+                            'fields': x['fields']
+                       },
+                       lambda y: {},
+                       supportFields=('jzrq', 'jjdm', 'jjjz', 'ljjz', 'hbcl','hbdr','fqdwjz')
+                       )
+    '''公募基金净值多基金代码'''
 
     # 公募基金回报
     FUND_JJHB = UrlCfg('http://%s/data/fund/brinson/jjhb', 'post', 'ams',
