@@ -356,11 +356,11 @@ class BL_Model:
         # get last 10 years trade month date
         trade_date_month = self.get_trade_date(end_date, fre='M')[-200:]
 
-        # last 6 years weekly data is needed for calculating the cov martix
+        # last 6 years fund_stats data is needed for calculating the cov martix
         begin_date = str(int(end_date[0:4]) - 6) + end_date[4:6] + '01'
         trade_date_week = self.get_trade_date(end_date=end_date, fre='W', begin_date=begin_date)
 
-        # calcualte the Cov matrix based on weekly return for the laset 6years
+        # calcualte the Cov matrix based on fund_stats return for the laset 6years
         cov_matrix = self.cov_mat_from_return(trade_date_week, asset_list, asset_type='index')
 
         # write cov_matrix into database
@@ -396,10 +396,10 @@ class BL_Model:
         #########################################for public fund asset #########################################
         # for index asset
 
-        #last 3.5 years/ 182 weeks  weekly data is needed for calculating the cov martix
+        #last 3.5 years/ 182 weeks  fund_stats data is needed for calculating the cov martix
         trade_date_week=self.get_trade_date(end_date=end_date,fre='W')[-(182+1):]
 
-        #calcualte the Cov matrix based on weekly return for the laset  3.5 years/ 182 weeks
+        #calcualte the Cov matrix based on fund_stats return for the laset  3.5 years/ 182 weeks
         cov_matrix=self.cov_mat_from_return(trade_date_week,asset_list,asset_type='public_fund')
 
         # write cov_matrix into database
@@ -431,13 +431,13 @@ class BL_Model:
 if __name__ == '__main__':
 
 
-        end_date='20220430'
-        version='20220430_v1'
+        end_date='20220831'
+        version='20220830_v1'
         blm=BL_Model()
         localdb_engine=db_engine.PrvFunDB().engine
 
-        blm.update_valuation_data()
-        # blm.update_asset_pool(r"E:\GitFolder\hbshare\fe\AAM\blmodel\asset_pool_update.xls", version)
+        #blm.update_asset_pool(r"E:\GitFolder\hbshare\fe\AAM\blmodel\asset_pool_update.xls", version)
+        #blm.update_valuation_data()
         #blm.update_marketvalue_data()
 
 
