@@ -7,7 +7,8 @@ import pandas as pd
 
 
 class MaxReturn:
-    def __init__(self, asset_list, ret, cov, target_risk, lb_list, ub_list, total_weight):
+    def __init__(self, asset_type, asset_list, ret, cov, target_risk, lb_list, ub_list, asset_category_df, category_lb_list, category_ub_list, total_weight):
+        self.asset_type = asset_type
         self.asset_list = asset_list
         self.ret = np.matrix(ret)
         self.cov = np.matrix(cov)
@@ -15,17 +16,35 @@ class MaxReturn:
         self.lb_list = lb_list
         self.ub_list = ub_list
         self.total_weight = total_weight
+        if self.asset_type == 'fund':
+            self.asset_category = asset_category_df.reset_index()
+            self.asset_category['IS_CATEGORY'] = 1
+            self.asset_category = self.asset_category.pivot(index='CATEGORY', columns='index', values='IS_CATEGORY')
+            self.asset_category = self.asset_category[self.asset_list].reindex(['EQUITY', 'BOND', 'COMMODITY', 'CURRENCY']).fillna(0)
+            self.asset_category = np.matrix(self.asset_category)
+            self.category_lb_list = category_lb_list
+            self.category_ub_list = category_ub_list
 
     def solve(self):
         n = len(self.asset_list)
         w = cp.Variable(n)
         ret = self.ret.T @ w
         risk = cp.quad_form(w, self.cov)
-        prob = cp.Problem(cp.Maximize(ret),
-                          [risk <= self.target_risk ** 2.0,
-                           cp.sum(w) == self.total_weight,
-                           w >= self.lb_list,
-                           w <= self.ub_list])
+        if self.asset_type == 'fund':
+            category_cons = self.asset_category @ w
+            prob = cp.Problem(cp.Maximize(ret),
+                              [risk <= self.target_risk ** 2.0,
+                               cp.sum(w) == self.total_weight,
+                               w >= self.lb_list,
+                               w <= self.ub_list,
+                               category_cons >= self.category_lb_list,
+                               category_cons <= self.category_ub_list])
+        else:
+            prob = cp.Problem(cp.Maximize(ret),
+                              [risk <= self.target_risk ** 2.0,
+                               cp.sum(w) == self.total_weight,
+                               w >= self.lb_list,
+                               w <= self.ub_list])
         prob.solve()
         if prob.status != 'optimal':
             weight = pd.Series(index=self.asset_list + ['cash'], data=[np.nan] * (len(self.asset_list) + 1))
@@ -47,7 +66,8 @@ class MaxReturn:
 
 
 class MinRisk:
-    def __init__(self, asset_list, ret, cov, target_return, lb_list, ub_list, total_weight):
+    def __init__(self, asset_type, asset_list, ret, cov, target_return, lb_list, ub_list, asset_category_df, category_lb_list, category_ub_list, total_weight):
+        self.asset_type = asset_type
         self.asset_list = asset_list
         self.ret = np.matrix(ret)
         self.cov = np.matrix(cov)
@@ -55,17 +75,35 @@ class MinRisk:
         self.lb_list = lb_list
         self.ub_list = ub_list
         self.total_weight = total_weight
+        if self.asset_type == 'fund':
+            self.asset_category = asset_category_df.reset_index()
+            self.asset_category['IS_CATEGORY'] = 1
+            self.asset_category = self.asset_category.pivot(index='CATEGORY', columns='index', values='IS_CATEGORY')
+            self.asset_category = self.asset_category[self.asset_list].reindex(['EQUITY', 'BOND', 'COMMODITY', 'CURRENCY']).fillna(0)
+            self.asset_category = np.matrix(self.asset_category)
+            self.category_lb_list = category_lb_list
+            self.category_ub_list = category_ub_list
 
     def solve(self):
         n = len(self.asset_list)
         w = cp.Variable(n)
         ret = self.ret.T @ w
         risk = cp.quad_form(w, self.cov)
-        prob = cp.Problem(cp.Minimize(risk),
-                          [ret >= self.target_return,
-                           cp.sum(w) == self.total_weight,
-                           w >= self.lb_list,
-                           w <= self.ub_list])
+        if self.asset_type == 'fund':
+            category_cons = self.asset_category @ w
+            prob = cp.Problem(cp.Minimize(risk),
+                              [ret >= self.target_return,
+                               cp.sum(w) == self.total_weight,
+                               w >= self.lb_list,
+                               w <= self.ub_list,
+                               category_cons >= self.category_lb_list,
+                               category_cons <= self.category_ub_list])
+        else:
+            prob = cp.Problem(cp.Minimize(risk),
+                              [ret >= self.target_return,
+                               cp.sum(w) == self.total_weight,
+                               w >= self.lb_list,
+                               w <= self.ub_list])
         prob.solve()
         if prob.status != 'optimal':
             weight = pd.Series(index=self.asset_list + ['cash'], data=[np.nan] * (len(self.asset_list) + 1))
@@ -87,7 +125,8 @@ class MinRisk:
 
 
 class MaxUtility:
-    def __init__(self, asset_list, ret, cov, risk_aversion, lb_list, ub_list, total_weight):
+    def __init__(self, asset_type, asset_list, ret, cov, risk_aversion, lb_list, ub_list, asset_category_df, category_lb_list, category_ub_list, total_weight):
+        self.asset_type = asset_type
         self.asset_list = asset_list
         self.ret = np.matrix(ret)
         self.cov = np.matrix(cov)
@@ -95,17 +134,33 @@ class MaxUtility:
         self.lb_list = lb_list
         self.ub_list = ub_list
         self.total_weight = total_weight
+        if self.asset_type == 'fund':
+            self.asset_category = asset_category_df.reset_index()
+            self.asset_category['IS_CATEGORY'] = 1
+            self.asset_category = self.asset_category.pivot(index='CATEGORY', columns='index', values='IS_CATEGORY')
+            self.asset_category = self.asset_category[self.asset_list].reindex(['EQUITY', 'BOND', 'COMMODITY', 'CURRENCY']).fillna(0)
+            self.asset_category = np.matrix(self.asset_category)
+            self.category_lb_list = category_lb_list
+            self.category_ub_list = category_ub_list
 
     def solve(self):
         n = len(self.asset_list)
         w = cp.Variable(n)
         ret = self.ret.T @ w
         risk = cp.quad_form(w, self.cov)
-        prob = cp.Problem(cp.Maximize(ret - 0.5 * self.risk_aversion * risk),
-                          [cp.sum(w) == self.total_weight,
-                           w >= self.lb_list,
-                           w <= self.ub_list])
-        prob.is_dcp()
+        if self.asset_type == 'fund':
+            category_cons = self.asset_category @ w
+            prob = cp.Problem(cp.Maximize(ret - 0.5 * self.risk_aversion * risk),
+                              [cp.sum(w) == self.total_weight,
+                               w >= self.lb_list,
+                               w <= self.ub_list,
+                               category_cons >= self.category_lb_list,
+                               category_cons <= self.category_ub_list])
+        else:
+            prob = cp.Problem(cp.Maximize(ret - 0.5 * self.risk_aversion * risk),
+                              [cp.sum(w) == self.total_weight,
+                               w >= self.lb_list,
+                               w <= self.ub_list])
         prob.solve()
         if prob.status != 'optimal':
             weight = pd.Series(index=self.asset_list + ['cash'], data=[np.nan] * (len(self.asset_list) + 1))
@@ -127,22 +182,41 @@ class MaxUtility:
 
 
 class MSR:
-    def __init__(self, asset_list, ret, cov, lb_list, ub_list, total_weight):
+    def __init__(self, asset_type, asset_list, ret, cov, lb_list, ub_list, asset_category_df, category_lb_list, category_ub_list, total_weight):
+        self.asset_type = asset_type
         self.asset_list = asset_list
         self.ret = np.matrix(ret)
         self.cov = np.matrix(cov)
         self.lb_list = lb_list
         self.ub_list = ub_list
         self.total_weight = total_weight
+        if self.asset_type == 'fund':
+            self.asset_category = asset_category_df.reset_index()
+            self.asset_category['IS_CATEGORY'] = 1
+            self.asset_category = self.asset_category.pivot(index='CATEGORY', columns='index', values='IS_CATEGORY')
+            self.asset_category = self.asset_category[self.asset_list].reindex(['EQUITY', 'BOND', 'COMMODITY', 'CURRENCY']).fillna(0)
+            self.asset_category = np.matrix(self.asset_category)
+            self.category_lb_list = category_lb_list
+            self.category_ub_list = category_ub_list
 
     def solve(self):
         n = len(self.asset_list)
         w = cp.Variable(n)
         ret = self.ret.T @ w
         risk = cp.quad_form(w, self.cov)
-        prob = cp.Problem(cp.Minimize(risk),
-                          [ret == 1,
-                           w >= self.lb_list])
+        if self.asset_type == 'fund':
+            category_cons = self.asset_category @ w
+            prob = cp.Problem(cp.Minimize(risk),
+                              [ret == self.ret.mean(),
+                               w >= self.lb_list * cp.sum(w),
+                               w <= self.ub_list * cp.sum(w),
+                               category_cons >= self.category_lb_list * cp.sum(w),
+                               category_cons <= self.category_ub_list * cp.sum(w)])
+        else:
+            prob = cp.Problem(cp.Minimize(risk),
+                              [ret == self.ret.mean(),
+                               w >= self.lb_list * cp.sum(w),
+                               w <= self.ub_list * cp.sum(w)])
         prob.solve()
         if prob.status != 'optimal':
             weight = pd.Series(index=self.asset_list + ['cash'], data=[np.nan] * (len(self.asset_list) + 1))
