@@ -44,10 +44,12 @@ def correct_cov(cov):
     return cov
 
 class RiskParityBudget:
-    def __init__(self, asset_type, asset_list, method, start_date, end_date, is_reblance, reblance_type, n, frequency, compute_risk_days,
-                 lb_list=None, ub_list=None, total_weight=None, risk_budget_list=None, processes=16):
+    def __init__(self, asset_type, asset_list, asset_type_dict, method, start_date, end_date, is_reblance, reblance_type, n, frequency, compute_risk_days,
+                 lb_list=None, ub_list=None, asset_category_dict=None, category_lb_list=None, category_ub_list=None, total_weight=None, risk_budget_list=None, processes=16):
         self.asset_type = asset_type
         self.asset_list = asset_list
+        self.asset_type_dict = asset_type_dict
+        self.asset_type_df = pd.DataFrame.from_dict(self.asset_type_dict, orient='index', columns=['TYPE'])
         self.method = method
         self.start_date = start_date
         self.end_date = end_date
@@ -58,6 +60,10 @@ class RiskParityBudget:
         self.compute_risk_days = compute_risk_days
         self.lb_list = [0.0] * len(self.asset_list) if lb_list is None or lb_list == [] else lb_list
         self.ub_list = [1.0] * len(self.asset_list) if ub_list is None or ub_list == [] else ub_list
+        self.asset_category_dict = {} if asset_category_dict is None else asset_category_dict
+        self.asset_category_df = pd.DataFrame.from_dict(self.asset_category_dict, orient='index', columns=['CATEGORY'])
+        self.category_lb_list = [0.0, 0.0, 0.0, 0.0] if category_lb_list is None or category_lb_list == [] else category_lb_list
+        self.category_ub_list = [1.0, 1.0, 1.0, 1.0] if category_ub_list is None or category_ub_list == [] else category_ub_list
         self.total_weight = 1.0 if total_weight is None else total_weight
         self.risk_budget_list = [1.0 / len(self.asset_list)] * len(self.asset_list) if risk_budget_list is None or risk_budget_list == [] else risk_budget_list
         self.risk_budget_list = [budget / sum(self.risk_budget_list) for budget in self.risk_budget_list]
@@ -69,6 +75,8 @@ class RiskParityBudget:
     def _verify_input_param(self):
         verify_type(self.asset_type, 'asset_type', str)
         verify_type(self.asset_list, 'asset_list', list)
+        verify_type(self.asset_type_dict, 'asset_type_dict', dict)
+        verify_type(self.asset_type_df, 'asset_type_df', pd.DataFrame)
         verify_type(self.method, 'method', str)
         verify_type(self.start_date, 'start_date', str)
         verify_type(self.end_date, 'end_date', str)
@@ -79,6 +87,10 @@ class RiskParityBudget:
         verify_type(self.compute_risk_days, 'compute_risk_days', int)
         verify_type(self.lb_list, 'lb_list', list)
         verify_type(self.ub_list, 'ub_list', list)
+        verify_type(self.asset_category_dict, 'asset_category_dict', dict)
+        verify_type(self.asset_category_df, 'asset_category_df', pd.DataFrame)
+        verify_type(self.category_lb_list, 'category_lb_list', list)
+        verify_type(self.category_ub_list, 'category_ub_list', list)
         verify_type(self.total_weight, 'total_weight', float)
         verify_type(self.risk_budget_list, 'risk_budget_list', list)
         verify_type(self.processes, 'processes', int)
@@ -90,6 +102,10 @@ class RiskParityBudget:
             msg = "asset_list is empty, check your input"
             logger.error(msg)
             raise InputParameterError(message=msg)
+        if len(self.asset_type_dict) != len(self.asset_list):
+            msg = "the length of asset_type_dict is not the same with asset_list, check your input"
+            logger.error(msg)
+            raise InputParameterError(message=msg)
         if self.method not in ['risk_parity', 'risk_budget']:
             msg = "method not supported, check your input"
             logger.error(msg)
@@ -122,6 +138,22 @@ class RiskParityBudget:
             msg = "ub must be between 0 and 1, check your input"
             logger.error(msg)
             raise InputParameterError(message=msg)
+        if self.asset_type == 'fund' and len(self.asset_category_dict) != len(self.asset_list):
+            msg = "the length of asset_category_dict is not the same with asset_list, check your input"
+            logger.error(msg)
+            raise InputParameterError(message=msg)
+        if len(self.category_lb_list) != 4 or len(self.category_ub_list) != 4:
+            msg = "the length of category_lb_list or category_ub_list must be 4, check your input"
+            logger.error(msg)
+            raise InputParameterError(message=msg)
+        if len([i for i in self.category_lb_list if (i >= 0.0 and i <= 1.0)]) != 4:
+            msg = "category_lb_list must be between 0 and 1, check your input"
+            logger.error(msg)
+            raise InputParameterError(message=msg)
+        if len([i for i in self.category_ub_list if (i >= 0.0 and i <= 1.0)]) != 4:
+            msg = "category_ub_list must be between 0 and 1, check your input"
+            logger.error(msg)
+            raise InputParameterError(message=msg)
         if not (self.total_weight >= 0.0 and self.total_weight <= 1.0):
             msg = "total_weight must be between 0 and 1, check your input"
             logger.error(msg)
@@ -141,6 +173,8 @@ class RiskParityBudget:
 
     def _load(self):
         self.calendar_df = Loader().read_cal('19900101', self.end_date)
+        print('length of calendar_df: ', len(self.calendar_df))
+        print('calendar_df: ', self.calendar_df)
         self.calendar_df = self.calendar_df.rename(columns={'jyrq': 'CALENDAR_DATE', 'sfjj': 'IS_OPEN', 'sfzm': 'IS_WEEK_END', 'sfym': 'IS_MONTH_END'})
         self.calendar_df['CALENDAR_DATE'] = self.calendar_df['CALENDAR_DATE'].astype(str)
         self.calendar_df = self.calendar_df.sort_values('CALENDAR_DATE')
@@ -164,57 +198,43 @@ class RiskParityBudget:
         self.start_date_backup = self.trade_cal[self.trade_cal['TRADE_DATE'] <= self.start_date]['TRADE_DATE'].unique().tolist()[-self.window - 40]
 
         if self.asset_type == 'index':
-            self.mutual_index_nav_df = Loader().read_mutual_index_daily_k_given_indexs(self.asset_list, self.start_date_backup, self.end_date)
-            self.mutual_index_nav_df = self.mutual_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(self.mutual_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
-            self.mutual_index_nav_df = self.mutual_index_nav_df.drop_duplicates()
-            self.mutual_index_nav_df['TRADE_DATE'] = self.mutual_index_nav_df['TRADE_DATE'].astype(str)
-            self.mutual_index_nav_df = self.mutual_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
-            self.mutual_index_nav_df = self.mutual_index_nav_df.sort_index()
-
-            self.private_index_nav_df = Loader().read_private_index_daily_k_given_indexs(self.asset_list, self.start_date_backup, self.end_date)
-            self.private_index_nav_df = self.private_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(self.private_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
-            self.private_index_nav_df = self.private_index_nav_df.drop_duplicates()
-            self.private_index_nav_df['TRADE_DATE'] = self.private_index_nav_df['TRADE_DATE'].astype(str)
-            self.private_index_nav_df = self.private_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
-            self.private_index_nav_df = self.private_index_nav_df.sort_index()
-
-            market_index_nav_df_list = []
-            for index in self.asset_list:
-                market_index_nav_df = Loader().read_market_index_daily_k_given_index(index, self.start_date_backup, self.end_date)
-                market_index_nav_df = market_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(market_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
-                market_index_nav_df_list.append(market_index_nav_df)
-            self.market_index_nav_df = pd.concat(market_index_nav_df_list)
-            self.market_index_nav_df = self.market_index_nav_df.drop_duplicates()
-            self.market_index_nav_df['TRADE_DATE'] = self.market_index_nav_df['TRADE_DATE'].astype(str)
-            self.market_index_nav_df = self.market_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
-            self.market_index_nav_df = self.market_index_nav_df.sort_index()
-
-            self.nav_df = pd.concat([self.mutual_index_nav_df, self.private_index_nav_df, self.market_index_nav_df], axis=1)
+            group_size = 10
+            group = int(len(self.asset_type_df) / group_size) + 1 if len(self.asset_type_df) % group_size > 0 else int(len(self.asset_type_df) / group_size) + 0
+            nav_df_list = []
+            for i in range(group):
+                asset_type_df = self.asset_type_df.iloc[i * group_size: (i + 1) * group_size]
+                mutual_indexs = asset_type_df[asset_type_df['TYPE'] == 'MUTUAL_INDEX'].index.unique().tolist()
+                private_indexs = asset_type_df[asset_type_df['TYPE'] == 'PRIVATE_INDEX'].index.unique().tolist()
+                market_indexs = asset_type_df[asset_type_df['TYPE'] == 'MARKET_INDEX'].index.unique().tolist()
+                nav_df_group = Loader().read_index_daily_k_given_indexs(mutual_indexs=mutual_indexs, private_indexs=private_indexs, market_indexs=market_indexs, start=self.start_date_backup, end=self.end_date)
+                nav_df_group = nav_df_group[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(nav_df_group) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
+                nav_df_list.append(nav_df_group)
+            self.nav_df = pd.concat(nav_df_list)
+            print('length of nav_df: ', len(self.nav_df))
+            print('nav_df: ', self.nav_df)
+            self.nav_df = self.nav_df.drop_duplicates()
+            self.nav_df['TRADE_DATE'] = self.nav_df['TRADE_DATE'].astype(str)
+            self.nav_df = self.nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
+            self.nav_df = self.nav_df.sort_index()
 
         if self.asset_type == 'fund':
-            mutual_fund_nav_df_list = []
-            for code in self.asset_list:
-                mutual_fund_nav_df = Loader().read_mutual_fund_cumret_given_code(code, self.start_date_backup, self.end_date)
-                mutual_fund_nav_df = mutual_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(mutual_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
-                mutual_fund_nav_df_list.append(mutual_fund_nav_df)
-            self.mutual_fund_nav_df = pd.concat(mutual_fund_nav_df_list)
-            self.mutual_fund_nav_df = self.mutual_fund_nav_df.drop_duplicates()
-            self.mutual_fund_nav_df['TRADE_DATE'] = self.mutual_fund_nav_df['TRADE_DATE'].astype(str)
-            self.mutual_fund_nav_df = self.mutual_fund_nav_df.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
-            self.mutual_fund_nav_df = self.mutual_fund_nav_df.sort_index()
-
-            private_fund_nav_df_list = []
-            for code in self.asset_list:
-                private_fund_nav_df = Loader().read_private_fund_adj_nav_given_code(code, self.start_date_backup, self.end_date)
-                private_fund_nav_df = private_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(private_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
-                private_fund_nav_df_list.append(private_fund_nav_df)
-            self.private_fund_nav_df = pd.concat(private_fund_nav_df_list)
-            self.private_fund_nav_df = self.private_fund_nav_df.drop_duplicates()
-            self.private_fund_nav_df['TRADE_DATE'] = self.private_fund_nav_df['TRADE_DATE'].astype(str)
-            self.private_fund_nav_df = self.private_fund_nav_df.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
-            self.private_fund_nav_df = self.private_fund_nav_df.sort_index()
-
-            self.nav_df = pd.concat([self.mutual_fund_nav_df, self.private_fund_nav_df], axis=1)
+            group_size = 10
+            group = int(len(self.asset_type_df) / group_size) + 1 if len(self.asset_type_df) % group_size > 0 else int(len(self.asset_type_df) / group_size) + 0
+            nav_df_list = []
+            for i in range(group):
+                asset_type_df = self.asset_type_df.iloc[i * group_size: (i + 1) * group_size]
+                mutual_codes = asset_type_df[asset_type_df['TYPE'] == 'MUTUAL_FUND'].index.unique().tolist()
+                private_codes = asset_type_df[asset_type_df['TYPE'] == 'PRIVATE_FUND'].index.unique().tolist()
+                nav_df_group = Loader().read_fund_cumret_given_codes(mutual_codes=mutual_codes, private_codes=private_codes, start=self.start_date_backup, end=self.end_date)
+                nav_df_group = nav_df_group[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(nav_df_group) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
+                nav_df_list.append(nav_df_group)
+            self.nav_df = pd.concat(nav_df_list)
+            print('length of nav_df: ', len(self.nav_df))
+            print('nav_df: ', self.nav_df)
+            self.nav_df = self.nav_df.drop_duplicates()
+            self.nav_df['TRADE_DATE'] = self.nav_df['TRADE_DATE'].astype(str)
+            self.nav_df = self.nav_df.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
+            self.nav_df = self.nav_df.sort_index()
 
         nan_nav_list = [asset for asset in self.asset_list if asset not in self.nav_df.columns]
         if len(nan_nav_list) == 0:
@@ -270,10 +290,10 @@ class RiskParityBudget:
 
         # 确定每个再平衡时点所需数据
         trading_day_list = self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['TRADE_DATE'].unique().tolist()
-        if self.asset_type == 'index' and len(self.private_index_nav_df) > 0:
+        if self.asset_type == 'index' and len(self.asset_type_df[self.asset_type_df['TYPE'] == 'PRIVATE_INDEX']) > 0:
             date_list = self.calendar_df[self.calendar_df['IS_WEEK_END'] == 1]['TRADE_DATE'].unique().tolist()
             self.q = 52
-        elif self.asset_type == 'fund' and len(self.private_fund_nav_df) > 0:
+        elif self.asset_type == 'fund' and len(self.asset_type_df[self.asset_type_df['TYPE'] == 'PRIVATE_FUND']) > 0:
             date_list = self.calendar_df[self.calendar_df['IS_WEEK_END'] == 1]['TRADE_DATE'].unique().tolist()
             self.q = 52
         else:
@@ -324,13 +344,13 @@ class RiskParityBudget:
             try:
                 if self.method == 'risk_parity':
                     self.cov = period_data['cov_df']
-                    weight, status = RiskParity(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight).solve()
+                    weight, status = RiskParity(self.asset_type, self.asset_list, self.cov, self.lb_list, self.ub_list, self.asset_category_df, self.category_lb_list, self.category_ub_list, self.total_weight).solve()
                     weight = pd.DataFrame(weight, columns=[date]).T
                     reason = '{0}：模型当期无优化解！'.format(date) if status == 'infeasible' else np.nan
                     status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
                 else:
                     self.cov = period_data['cov_df']
-                    weight, status = RiskBudget(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight, self.risk_budget_list).solve()
+                    weight, status = RiskBudget(self.asset_type, self.asset_list, self.cov, self.lb_list, self.ub_list, self.asset_category_df, self.category_lb_list, self.category_ub_list, self.total_weight, self.risk_budget_list).solve()
                     weight = pd.DataFrame(weight, columns=[date]).T
                     reason = '{0}：模型当期无优化解！'.format(date) if status == 'infeasible' else np.nan
                     status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
@@ -379,8 +399,9 @@ if __name__ == '__main__':
     # mutual_fund: ['002943', '688888', '000729']
     # private_fund: ['SGK768', 'SX8958', 'SR4480']
 
-    weight_df, status_df = RiskParityBudget(asset_type='index',  # index, fund
-                                            asset_list=['000300', '000906', 'CBA00301'],
+    weight_df, status_df = RiskParityBudget(asset_type='fund',  # index, fund
+                                            asset_list=['002943', '688888', '000729'],
+                                            asset_type_dict={'002943': 'MUTUAL_FUND', '688888': 'MUTUAL_FUND', '000729': 'MUTUAL_FUND'},  # MUTUAL_INDEX, PRIVATE_INDEX, MARKET_INDEX, MUTUAL_FUND, PRIVATE_FUND
                                             method='risk_parity',  # risk_parity, risk_budget
                                             start_date='20181231',
                                             end_date='20220704',
@@ -389,6 +410,9 @@ if __name__ == '__main__':
                                             n=3,
                                             frequency='month',  # day, week, month, quarter, year
                                             compute_risk_days=120,  # 交易日
+                                            asset_category_dict={'002943': 'EQUITY', '688888': 'EQUITY', '000729': 'BOND'},  # EQUITY, BOND, COMMODITY, CURRENCY
+                                            category_lb_list=[0.0, 0.0, 0.0, 0.0],
+                                            category_ub_list=[0.5, 0.5, 1.0, 1.0],
                                             processes=4
                                             ).get_all()
     print(weight_df, status_df)
\ No newline at end of file
