@@ -1,17 +1,25 @@
 # -*- coding: utf-8 -*-
 
-from scipy.optimize import minimize
+from scipy.optimize import minimize, LinearConstraint
 import numpy as np
 import pandas as pd
 
 
 class RiskParity:
-    def __init__(self, asset_list, cov, lb_list, ub_list, total_weight):
+    def __init__(self, asset_type, asset_list, cov, lb_list, ub_list, asset_category_df, category_lb_list, category_ub_list, total_weight):
+        self.asset_type = asset_type
         self.asset_list = asset_list
         self.omega = np.matrix(cov)
         self.lb_list = lb_list
         self.ub_list = ub_list
         self.total_weight = total_weight
+        if self.asset_type == 'fund':
+            self.asset_category = asset_category_df.reset_index()
+            self.asset_category['IS_CATEGORY'] = 1
+            self.asset_category = self.asset_category.pivot(index='CATEGORY', columns='index', values='IS_CATEGORY')
+            self.asset_category = self.asset_category[self.asset_list].reindex(['EQUITY', 'BOND', 'COMMODITY', 'CURRENCY']).fillna(0)
+            self.category_lb_list = category_lb_list
+            self.category_ub_list = category_ub_list
 
     def _target_func(self, x):
         tmp = np.sqrt((np.matrix(x) * self.omega * np.matrix(x).T).A1)
@@ -20,7 +28,11 @@ class RiskParity:
         return delta_risk
 
     def _get_cons(self):
-        cons = ({'type': 'eq', 'fun': lambda x: sum(x) - self.total_weight})
+        if self.asset_type == 'fund':
+            cons = ({'type': 'eq', 'fun': lambda x: sum(x) - self.total_weight},
+                    LinearConstraint(self.asset_category, lb=self.category_lb_list, ub=self.category_ub_list))
+        else:
+            cons = ({'type': 'eq', 'fun': lambda x: sum(x) - self.total_weight})
         return cons
 
     def _get_bnds(self):
@@ -55,13 +67,21 @@ class RiskParity:
 
 
 class RiskBudget:
-    def __init__(self, asset_list, cov, lb_list, ub_list, total_weight, risk_budget_list):
+    def __init__(self, asset_type, asset_list, cov, lb_list, ub_list, asset_category_df, category_lb_list, category_ub_list, total_weight, risk_budget_list):
+        self.asset_type = asset_type
         self.asset_list = asset_list
         self.omega = np.matrix(cov)
         self.lb_list = lb_list
         self.ub_list = ub_list
         self.total_weight = total_weight
         self.risk_budget_list = risk_budget_list
+        if self.asset_type == 'fund':
+            self.asset_category = asset_category_df.reset_index()
+            self.asset_category['IS_CATEGORY'] = 1
+            self.asset_category = self.asset_category.pivot(index='CATEGORY', columns='index', values='IS_CATEGORY')
+            self.asset_category = self.asset_category[self.asset_list].reindex(['EQUITY', 'BOND', 'COMMODITY', 'CURRENCY']).fillna(0)
+            self.category_lb_list = category_lb_list
+            self.category_ub_list = category_ub_list
 
     def _target_func(self, x):
         tmp = np.sqrt((np.matrix(x) * self.omega * np.matrix(x).T).A1)
@@ -71,7 +91,11 @@ class RiskBudget:
         return delta_risk
 
     def _get_cons(self):
-        cons = ({'type': 'eq', 'fun': lambda x: sum(x) - self.total_weight})
+        if self.asset_type == 'fund':
+            cons = ({'type': 'eq', 'fun': lambda x: sum(x) - self.total_weight},
+                    LinearConstraint(self.asset_category, lb=self.category_lb_list, ub=self.category_ub_list))
+        else:
+            cons = ({'type': 'eq', 'fun': lambda x: sum(x) - self.total_weight})
         return cons
 
     def _get_bnds(self):
