@@ -9,9 +9,11 @@ import pandas as pd
 
 
 class Simple:
-    def __init__(self, asset_type, asset_list, method, start_date, end_date, is_reblance, reblance_type, n, frequency, set_w=None):
+    def __init__(self, asset_type, asset_list, asset_type_dict, method, start_date, end_date, is_reblance, reblance_type, n, frequency, set_w=None):
         self.asset_type = asset_type
         self.asset_list = asset_list
+        self.asset_type_dict = asset_type_dict
+        self.asset_type_df = pd.DataFrame.from_dict(self.asset_type_dict, orient='index', columns=['TYPE'])
         self.method = method
         self.start_date = start_date
         self.end_date = end_date
@@ -28,6 +30,8 @@ class Simple:
     def _verify_input_param(self):
         verify_type(self.asset_type, 'asset_type', str)
         verify_type(self.asset_list, 'asset_list', list)
+        verify_type(self.asset_type_dict, 'asset_type_dict', dict)
+        verify_type(self.asset_type_df, 'asset_type_df', pd.DataFrame)
         verify_type(self.method, 'method', str)
         verify_type(self.start_date, 'start_date', str)
         verify_type(self.end_date, 'end_date', str)
@@ -44,6 +48,10 @@ class Simple:
             msg = "asset_list is empty, check your input"
             logger.error(msg)
             raise InputParameterError(message=msg)
+        if len(self.asset_type_df) != len(self.asset_list):
+            msg = "the length of asset_type is not match with asset_list"
+            logger.error(msg)
+            raise InputParameterError(message=msg)
         if self.method not in ['simple']:
             msg = "method not supported, check your input"
             logger.error(msg)
@@ -71,7 +79,7 @@ class Simple:
 
     def _load(self):
         self.calendar_df = Loader().read_cal('19900101', self.end_date)
-        self.calendar_df = self.calendar_df.rename(columns={'JYRQ': 'CALENDAR_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
+        self.calendar_df = self.calendar_df.rename(columns={'jyrq': 'CALENDAR_DATE', 'sfjj': 'IS_OPEN', 'sfzm': 'IS_WEEK_END', 'sfym': 'IS_MONTH_END'})
         self.calendar_df['CALENDAR_DATE'] = self.calendar_df['CALENDAR_DATE'].astype(str)
         self.calendar_df = self.calendar_df.sort_values('CALENDAR_DATE')
         self.calendar_df['IS_OPEN'] = self.calendar_df['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
@@ -165,6 +173,7 @@ if __name__ == '__main__':
 
     weight_df, status_df = Simple(asset_type='index',  # index, fund
                                   asset_list=['000300', '000906', 'CBA00301'],
+                                  asset_type_dict={'000300': 'MARKET_INDEX', '000906': 'MARKET_INDEX', 'CBA00301': 'MARKET_INDEX'},  # MUTUAL_INDEX, PRIVATE_INDEX, MARKET_INDEX, MUTUAL_FUND, PRIVATE_FUND
                                   method='simple',  # simple
                                   start_date='20181231',
                                   end_date='20220704',
