@@ -150,18 +150,17 @@ class Crowding:
         index_data = index_data.sort_values('TRADE_DATE')
         index_data['IDX'] = range(len(index_data))
         for col in list(index_data.columns)[3:]:
-            index_data[col] = index_data['IDX'].rolling(window=250, min_periods=250, center=False).apply(lambda x: self.quantile_definition(col, x, index_data))
-        index_data['CROWDING'] = index_data[list(index_data.columns)[3:]].mean(axis=1)
-        index_data = index_data.dropna(subset=['CROWDING'])
-        index_data['CROWDING_MEAN'] = index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).mean()
-        index_data['CROWDING_UP1'] = index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).mean() + 1.0 * index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
-        index_data['CROWDING_DOWN1'] = index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).mean() - 1.0 * index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
-        index_data['CROWDING_UP15'] = index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).mean() + 1.5 * index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
-        index_data['CROWDING_DOWN15'] = index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).mean() - 1.5 * index_data['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
-
+            index_data[col + '_Q'] = index_data['IDX'].rolling(window=250, min_periods=250, center=False).apply(lambda x: self.quantile_definition(col, x, index_data))
+        index_data = index_data.drop(['IDX', 'IDX_Q'], axis=1)
+        index_data['CROWDING'] = index_data[list(index_data.columns)[-5:]].mean(axis=1)
+        index_data['CROWDING_MEAN'] = index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).mean()
+        index_data['CROWDING_UP1'] = index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).mean() + 1.0 * index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).std(ddof=1)
+        index_data['CROWDING_DOWN1'] = index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).mean() - 1.0 * index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).std(ddof=1)
+        index_data['CROWDING_UP15'] = index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).mean() + 1.5 * index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).std(ddof=1)
+        index_data['CROWDING_DOWN15'] = index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).mean() - 1.5 * index_data['CROWDING'].rolling(window=250, min_periods=250, center=False).std(ddof=1)
+        index_data = index_data.dropna()
         index_data_disp = index_data.merge(self.trade_df[['TRADE_DATE', 'IS_WEEK_END']], on=['TRADE_DATE'], how='left')
         index_data_disp = index_data_disp[index_data_disp['IS_WEEK_END'] == '1']
-        index_data_disp = index_data_disp[index_data_disp['TRADE_DATE'] >= '20200101']
         index_data_disp['TRADE_DATE'] = index_data_disp['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
         fig, ax = plt.subplots(figsize=(12, 6))
         ax1 = ax
