@@ -86,7 +86,7 @@ def holding_operation(stock, fund_code, fund_start_date=None):
     ax2.legend(loc=1)
     ax2.yaxis.set_major_formatter(FuncFormatter(to_percent))
     plt.tight_layout()
-    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/{1}.png'.format(fund_code, stock_name))
+    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/{1}.png'.format(fund_code, stock_name))
     # plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/{0}_{1}.png'.format(fund_name, stock_name))
     return
 
@@ -236,6 +236,8 @@ class HoldingCompare:
             data.append(stock_finance_date)
             print('[PreloadFinance][{0}/{1}]'.format(idx, len(report_df['REPORT_DATE'])))
         self.all_stock_finance = pd.concat([existed_stock_finance] + data, ignore_index=True)
+        self.all_stock_finance = self.all_stock_finance.loc[self.all_stock_finance['TICKER_SYMBOL'].str.len() == 6]
+        self.all_stock_finance = self.all_stock_finance.loc[self.all_stock_finance['TICKER_SYMBOL'].astype(str).str.slice(0, 1).isin(['0', '3', '6'])]
         self.all_stock_finance.to_hdf(stock_finance_path, key='table', mode='w')
         # self.all_stock_finance = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/stock_finance.hdf', key='table')
         # 行业行情数据
@@ -353,8 +355,8 @@ class HoldingCompare:
                     ax1_right.legend(loc=1)
                     ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                     plt.tight_layout()
-                    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/{1}_{2}.png'.format(self.target_fund_code, stock_name, target_fund_name))
-                    worksheet_h.insert_image('A{0}'.format(idx + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/{1}_{2}.png'.format(self.target_fund_code, stock_name, target_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
+                    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding/{1}_{2}.png'.format(self.target_fund_code, stock_name, target_fund_name))
+                    worksheet_h.insert_image('A{0}'.format(idx + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding/{1}_{2}.png'.format(self.target_fund_code, stock_name, target_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                     ########################################
                     hypre_link = r'=HYPERLINK("#' + '个股仓位变动图' + '!' + 'A{0}'.format(idx + 1) + '\",\"' + fund_holding.iloc[idx, 5] + '")'
                     fund_holding.iloc[idx, 5] = hypre_link
@@ -474,8 +476,8 @@ class HoldingCompare:
                 ax1_right.legend(loc=1)
                 ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                 plt.tight_layout()
-                plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, stock_name, target_fund_name, compare_fund_name))
-                worksheet_hc.insert_image('{0}{1}'.format(cell_dic[j + 1], i + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, stock_name, target_fund_name, compare_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
+                plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, stock_name, target_fund_name, compare_fund_name))
+                worksheet_hc.insert_image('{0}{1}'.format(cell_dic[j + 1], i + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, stock_name, target_fund_name, compare_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                 ########################################
 
                 stock_holding.loc[(stock_holding.index.str.slice(4, 6).isin(['06', '12'])) & (stock_holding[target_fund_name].isnull()), target_fund_name] = 0.00
@@ -610,8 +612,8 @@ class HoldingCompare:
                     # ax1_right.legend(loc=1)
                     # ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                     plt.tight_layout()
-                    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/{1}_{2}.png'.format(self.target_fund_code, industry, target_fund_name))
-                    worksheet_h.insert_image('A{0}'.format(idx + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/{1}_{2}.png'.format(self.target_fund_code, industry, target_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
+                    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding/{1}_{2}.png'.format(self.target_fund_code, industry, target_fund_name))
+                    worksheet_h.insert_image('A{0}'.format(idx + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding/{1}_{2}.png'.format(self.target_fund_code, industry, target_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                     ########################################
                     hypre_link = r'=HYPERLINK("#' + '行业仓位变动图' + '!' + 'A{0}'.format(idx + 1) + '\",\"' + fund_industry_holding.iloc[idx, 1] + '")'
                     fund_industry_holding.iloc[idx, 1] = hypre_link
@@ -726,8 +728,8 @@ class HoldingCompare:
                 # ax1_right.legend(loc=1)
                 # ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
                 plt.tight_layout()
-                plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, industry, target_fund_name, compare_fund_name))
-                worksheet_hc.insert_image('{0}{1}'.format(cell_dic[j + 1], i + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, industry, target_fund_name, compare_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
+                plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, industry, target_fund_name, compare_fund_name))
+                worksheet_hc.insert_image('{0}{1}'.format(cell_dic[j + 1], i + 1), 'D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding_compare/{1}_{2}_{3}.png'.format(self.target_fund_code, industry, target_fund_name, compare_fund_name), {'x_scale': 0.05, 'y_scale': 0.05})
                 ########################################
 
                 industry_holding[target_fund_name] = industry_holding[target_fund_name].fillna(0.00)
@@ -761,22 +763,99 @@ class HoldingCompare:
             industry_holding_compare = pd.concat([fund_industry_holding], axis=1)
         return industry_holding_compare
 
+    def get_stock_operation(self, stocks):
+        fund_name = self.all_fund[self.all_fund['FUND_CODE'] == self.target_fund_code]['FUND_NAME'].values[0]
+        for stock in stocks:
+            # 个股仓位变动图
+            stock_daily_k = self.all_stock_daily_k[self.all_stock_daily_k['TICKER_SYMBOL'] == stock]
+            stock_daily_k['TRADE_DATE'] = stock_daily_k['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
+            stock_daily_k = stock_daily_k.sort_values('TRADE_DATE')
+            stock_name = stock_daily_k['SEC_SHORT_NAME'].values[-1]
+            stock_valuation = self.all_stock_valuation[self.all_stock_valuation['TICKER_SYMBOL'] == stock]
+            stock_valuation['TRADE_DATE'] = stock_valuation['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
+            stock_valuation = stock_valuation.sort_values('TRADE_DATE')
+            stock_finance = self.all_stock_finance[self.all_stock_finance['TICKER_SYMBOL'] == stock]
+            stock_finance = stock_finance.sort_values(['END_DATE', 'PUBLISH_DATE']).drop_duplicates(['END_DATE'], keep='last')
+            stock_finance = stock_finance.merge(self.all_calendar_trade_df[['CALENDAR_DATE', 'TRADE_DATE']].rename(columns={'CALENDAR_DATE': 'END_DATE'}), on=['END_DATE'], how='left')
+            stock_finance['TRADE_DATE'] = stock_finance['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
+            stock_finance = stock_finance.sort_values('TRADE_DATE')
+            stock_holding_disp = self.all_fund_holding[(self.all_fund_holding['FUND_CODE'] == self.target_fund_code) & (self.all_fund_holding['TICKER_SYMBOL'] == stock)]
+            stock_holding_disp = stock_holding_disp[['REPORT_DATE', 'MV_IN_NA']].set_index('REPORT_DATE')
+            stock_holding_disp['MV_IN_NA'] = stock_holding_disp['MV_IN_NA'].apply(lambda x: round(x, 2))
+            stock_holding_disp.columns = [fund_name]
+            disp_start_date = min(stock_holding_disp.index)
+            disp_dates = self.all_report_df[self.all_report_df['REPORT_DATE'] >= disp_start_date]['REPORT_DATE'].unique().tolist()[:-1]
+            stock_holding_disp = stock_holding_disp.reindex(disp_dates)
+            stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['06', '12'])) & (stock_holding_disp[fund_name].isnull()), fund_name] = 0.00
+            stock_holding_disp = stock_holding_disp.merge(self.all_fund_holding_min[self.all_fund_holding_min['FUND_CODE'] == self.target_fund_code][['REPORT_DATE', 'MV_IN_NA_MIN']].set_index('REPORT_DATE'), left_index=True, right_index=True, how='left')
+            stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[fund_name].isnull()), fund_name] = stock_holding_disp.loc[(stock_holding_disp.index.str.slice(4, 6).isin(['03', '09'])) & (stock_holding_disp[fund_name].isnull()), 'MV_IN_NA_MIN']
+            stock_holding_disp = stock_holding_disp.drop('MV_IN_NA_MIN', axis=1)
+            stock_holding_disp[fund_name] = stock_holding_disp[fund_name].astype(str)
+            stock_holding_disp = stock_holding_disp.reset_index()
+            stock_holding_disp = stock_holding_disp.merge(self.all_calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
+            stock_holding_disp['TRADE_DATE'] = stock_holding_disp['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
+            stock_holding_disp = stock_holding_disp.sort_values('TRADE_DATE')
+            stock_holding_disp_quancang = stock_holding_disp.loc[stock_holding_disp['REPORT_DATE'].str.slice(4, 6).isin(['06', '12'])]
+            stock_holding_disp_quancang[fund_name] = stock_holding_disp_quancang[fund_name].replace('--', np.nan)
+            stock_holding_disp_quancang[fund_name] = stock_holding_disp_quancang[fund_name].astype(float)
+            stock_holding_disp_quancang = stock_holding_disp_quancang.rename(columns={fund_name: fund_name + '（全仓）'})
+            all_report_trade_df = self.all_report_trade_df[['TRADE_DATE']]
+            all_report_trade_df['TRADE_DATE'] = all_report_trade_df['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
+            all_report_trade_df['IS_QUARTER_END'] = 1
+
+            data = stock_daily_k[stock_daily_k['TRADE_DATE'] >= datetime.strptime(disp_start_date, '%Y%m%d') - timedelta(30)][['TRADE_DATE', 'CLOSE_PRICE']]
+            data = data.merge(stock_valuation[['TRADE_DATE', 'PE_TTM']], on=['TRADE_DATE'], how='left')
+            data = data.merge(stock_finance[['TRADE_DATE', 'ROE_TTM']], on=['TRADE_DATE'], how='left')
+            data = data.merge(stock_holding_disp[['TRADE_DATE', fund_name]], on=['TRADE_DATE'], how='left')
+            data = data.merge(stock_holding_disp_quancang[['TRADE_DATE', fund_name + '（全仓）']], on=['TRADE_DATE'], how='left')
+            data = data.merge(all_report_trade_df[['TRADE_DATE', 'IS_QUARTER_END']], on=['TRADE_DATE'], how='left')
+            data['INDEX'] = range(len(data))
+            data['TRADE_DATE_DISP'] = data['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d')[:6])
+            fig, ax = plt.subplots(2, 1, figsize=(25, 10))
+            bar_width = len(data) / len(data.dropna(subset=[fund_name])) / 10.0
+            ax0_right = ax[0].twinx()
+            ax[0].plot(data['INDEX'], data['CLOSE_PRICE'], color='#AFDEFA', label='{0}（元）'.format(stock_name))
+            ax0_right.bar(data['INDEX'], data[fund_name + '（全仓）'], width=bar_width, color='#C94649', label='占净值比（%）')
+            ax0_right.set_ylim([0, 50])
+            ax[0].scatter(data.dropna(subset=[fund_name])['INDEX'], data.dropna(subset=[fund_name])['CLOSE_PRICE'], marker='.', color='#F04950', label='{0}中该个股持仓占净值比（%）'.format(fund_name))
+            for x, y, value in zip(list(data.dropna(subset=[fund_name])['INDEX']), list(data.dropna(subset=[fund_name])['CLOSE_PRICE']), list(data.dropna(subset=[fund_name])[fund_name])):
+                ax[0].text(x, y * 1.1, value, ha='center', va='bottom', color='#F04950')
+            plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
+            ax[0].set_xlabel('')
+            ax[0].set_ylabel('')
+            ax0_right.set_ylabel('')
+            ax[0].legend(loc=2)
+            ax0_right.legend(loc=1)
+            ax0_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
+            ax1_right = ax[1].twinx()
+            ax[1].bar([i - bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['PE_TTM']), width=bar_width, color='#C94649', label='PE_TTM')
+            ax1_right.bar([i + bar_width / 2.0 for i in list(data.dropna(subset=['ROE_TTM'])['INDEX'])], list(data.dropna(subset=['ROE_TTM'])['ROE_TTM']), width=bar_width, color='#8588B7', label='ROE_TTM')
+            plt.xticks(ticks=data.dropna(subset=['IS_QUARTER_END'])['INDEX'], labels=data.dropna(subset=['IS_QUARTER_END'])['TRADE_DATE_DISP'], rotation=90)
+            ax[1].set_xlabel('')
+            ax[1].set_ylabel('')
+            ax1_right.set_ylabel('')
+            ax[1].legend(loc=2)
+            ax1_right.legend(loc=1)
+            ax1_right.yaxis.set_major_formatter(FuncFormatter(to_percent))
+            plt.tight_layout()
+            plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/{0}_{1}.png'.format(stock_name, fund_name))
+        return
 
     def get_holding_compare(self):
         if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/'):
             os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/holding_compare/')
-        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/'.format(self.target_fund_code)):
-            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/'.format(self.target_fund_code))
-        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/'.format(self.target_fund_code)):
-            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding/'.format(self.target_fund_code))
-        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/'.format(self.target_fund_code)):
-            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/stock_holding_compare/'.format(self.target_fund_code))
-        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/'.format(self.target_fund_code)):
-            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding/'.format(self.target_fund_code))
-        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/'.format(self.target_fund_code)):
-            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/industry_holding_compare/'.format(self.target_fund_code))
+        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/'.format(self.target_fund_code)):
+            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/'.format(self.target_fund_code))
+        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding/'.format(self.target_fund_code)):
+            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding/'.format(self.target_fund_code))
+        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding_compare/'.format(self.target_fund_code)):
+            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/stock_holding_compare/'.format(self.target_fund_code))
+        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding/'.format(self.target_fund_code)):
+            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding/'.format(self.target_fund_code))
+        if not os.path.exists('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding_compare/'.format(self.target_fund_code)):
+            os.makedirs('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/industry_holding_compare/'.format(self.target_fund_code))
 
-        excel_writer = pd.ExcelWriter('D:/Git/hbshare/hbshare/fe/xwq/data/quantitative_report/{0}/持仓情况.xlsx'.format(self.target_fund_code), engine='xlsxwriter')
+        excel_writer = pd.ExcelWriter('D:/Git/hbshare/hbshare/fe/xwq/data/fund_report/{0}/持仓情况.xlsx'.format(self.target_fund_code), engine='xlsxwriter')
         worksheet1 = excel_writer.book.add_worksheet('个股仓位变动图')
         worksheet2 = excel_writer.book.add_worksheet('个股仓位变动比较图')
         worksheet3 = excel_writer.book.add_worksheet('行业仓位变动图')
@@ -786,6 +865,7 @@ class HoldingCompare:
         stock_holding_compare.to_excel(excel_writer, sheet_name='个股持仓情况', index=False)
         industry_holding_compare.to_excel(excel_writer, sheet_name='行业持仓情况', index=False)
         excel_writer.save()
+        # self.get_stock_operation(['600745', '002049', '002920', '300474', '601012', '600438', '002250', '600406', '600703', '600021','600116'])
         return
 
 if __name__ == '__main__':
