@@ -68,17 +68,35 @@ def get_factor_data_df():
     factor_data_df = factor_data_df.merge(duration_data_df, left_index=True, right_index=True, how='left')
     return factor_data_df
 
+def get_fund_nav_series(fund_type, fund_id, start_date_backup, end_date):
+    if fund_type == 'mutual':
+        fund_nav_series = HBDB().read_fund_cumret_given_code(fund_id, start_date_backup, end_date)
+        if len(fund_nav_series) == 0:
+            fund_nav_series = pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'CUM_RET'])
+        fund_nav_series['TRADE_DATE'] = fund_nav_series['TRADE_DATE'].astype(str)
+        fund_nav_series = fund_nav_series.sort_values('TRADE_DATE')
+        fund_nav_series['ADJ_NAV'] = 0.01 * fund_nav_series['CUM_RET'] + 1
+        fund_nav_series = fund_nav_series.set_index('TRADE_DATE')
+        fund_nav_series = fund_nav_series['ADJ_NAV']
+    else:
+        fund_nav_series = HBDB().read_private_fund_cumret_given_code(fund_id, start_date_backup, end_date)
+        if len(fund_nav_series) == 0:
+            fund_nav_series = pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
+        fund_nav_series['TRADE_DATE'] = fund_nav_series['TRADE_DATE'].astype(str)
+        fund_nav_series = fund_nav_series.sort_values('TRADE_DATE')
+        fund_nav_series = fund_nav_series.set_index('TRADE_DATE')
+        fund_nav_series = fund_nav_series['ADJ_NAV']
+    return fund_nav_series
+
 
 class BondCampisiAttribution:
-    def __init__(self, calendar_df, factor_data_df, fund_id, fund_type, start_date, end_date, frequency, backup_period='整个季度', convertible=True, equity=True):
+    def __init__(self, calendar_df, factor_data_df, fund_nav_series, fund_id, fund_type, start_date, end_date, frequency, backup_period='整个季度', convertible=True, equity=True):
         """
         :param fund_id: 基金代码
         :param start_date: 归因的起始时间
         :param end_date: 归因的结束时间
         :param frequency: 归因频率
         """
-        self.calendar_df = calendar_df
-        self.factor_data_df = factor_data_df
         self.fund_id = fund_id
         self.fund_type = fund_type
         self.frequency = frequency
@@ -91,8 +109,11 @@ class BondCampisiAttribution:
         self.convertible = convertible
         self.equity = equity
         self.deposit = 0.015 / 365.0 if self.fund_type == 'mutual' else 0.015 / 52.0
+        self.calendar_df = calendar_df
+        self.calendar_df = self.calendar_df[self.calendar_df['CALENDAR_DATE'] <= self.end_date]
+        self.factor_data_df = factor_data_df
+        self.fund_nav_series = fund_nav_series
         self._verify_input_param()
-        self._load_data()
         self._init_bond_campisi_attribution_data()
 
     def _verify_input_param(self):
@@ -132,27 +153,6 @@ class BondCampisiAttribution:
             logger.error(msg)
             raise InputParameterError(message=msg)
 
-    def _load_data(self):
-        self.calendar_df = self.calendar_df[self.calendar_df['CALENDAR_DATE'] <= self.end_date]
-
-        if self.fund_type == 'mutual':
-            self.fund_nav_series = HBDB().read_fund_cumret_given_code(self.fund_id, self.start_date_backup, self.end_date)
-            if len(self.fund_nav_series) == 0:
-                self.fund_nav_series = pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'CUM_RET'])
-            self.fund_nav_series['TRADE_DATE'] = self.fund_nav_series['TRADE_DATE'].astype(str)
-            self.fund_nav_series = self.fund_nav_series.sort_values('TRADE_DATE')
-            self.fund_nav_series['ADJ_NAV'] = 0.01 * self.fund_nav_series['CUM_RET'] + 1
-            self.fund_nav_series = self.fund_nav_series.set_index('TRADE_DATE')
-            self.fund_nav_series = self.fund_nav_series['ADJ_NAV']
-        else:
-            self.fund_nav_series = HBDB().read_private_fund_cumret_given_code(self.fund_id, self.start_date_backup, self.end_date)
-            if len(self.fund_nav_series) == 0:
-                self.fund_nav_series = pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
-            self.fund_nav_series['TRADE_DATE'] = self.fund_nav_series['TRADE_DATE'].astype(str)
-            self.fund_nav_series = self.fund_nav_series.sort_values('TRADE_DATE')
-            self.fund_nav_series = self.fund_nav_series.set_index('TRADE_DATE')
-            self.fund_nav_series = self.fund_nav_series['ADJ_NAV']
-
     def get_trading_day_list(self, nav_frequency):
         if nav_frequency == 'day':
             trading_day_list = sorted(list(self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['CALENDAR_DATE']))
@@ -397,10 +397,16 @@ if __name__ == '__main__':
     bond_fund = bond_fund[bond_fund['BEGIN_DATE'] <= date_before]
 
     # 公募样例：
+    fund_type = 'mutual'
+    fund_id = '110008'
+    start_date = '20171201'
+    end_date = '20211231'
+    start_date_backup = (datetime.strptime(start_date, '%Y%m%d') - timedelta(2000)).strftime('%Y%m%d')
     calendar_df = get_calendar_df()
     factor_data_df = get_factor_data_df()
-    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='110008', fund_type='mutual', start_date='20171201', end_date='20211231', frequency='day', backup_period='近三月').get_all()
-    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='110008', fund_type='mutual', start_date='20171201', end_date='20211231', frequency='season').get_all()
+    fund_nav_series = get_fund_nav_series(fund_type, fund_id, start_date_backup, end_date)
+    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_nav_series, fund_id=fund_id, fund_type=fund_type, start_date=start_date, end_date=end_date, frequency='day', backup_period='近三月').get_all()
+    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_nav_series, fund_id=fund_id, fund_type=fund_type, start_date=start_date, end_date=end_date, frequency='season').get_all()
     print(res)
 
     # 分析样本：成立满6个月的债券型私募基金
@@ -408,6 +414,7 @@ if __name__ == '__main__':
     private_bond_fund = private_bond_fund.rename(columns={'jjdm': 'FUND_CODE', 'jjmc': 'FUND_FULL_NAME', 'jjjc': 'FUND_SHORT_NAME', 'clrq': 'BEGIN_DATE', 'zzrq': 'END_DATE', 'jjfl': 'FUND_TYPE_1ST'})
     private_bond_fund['FUND_TYPE_1ST'] = private_bond_fund['FUND_TYPE_1ST'].replace({'2': '债券型'})
     private_bond_fund = private_bond_fund.dropna(subset=['BEGIN_DATE'])
+    private_bond_fund.loc[private_bond_fund['END_DATE'] == '', 'END_DATE'] = np.nan
     private_bond_fund['END_DATE'] = private_bond_fund['END_DATE'].fillna(20990101)
     private_bond_fund['BEGIN_DATE'] = private_bond_fund['BEGIN_DATE'].astype(int)
     private_bond_fund['END_DATE'] = private_bond_fund['END_DATE'].astype(int)
@@ -417,10 +424,16 @@ if __name__ == '__main__':
     private_bond_fund = private_bond_fund[private_bond_fund['BEGIN_DATE'] <= date_before]
 
     # 私募样例：
+    fund_type = 'private'
+    fund_id = 'P11701'
+    start_date = '20171201'
+    end_date = '20211231'
+    start_date_backup = (datetime.strptime(start_date, '%Y%m%d') - timedelta(2000)).strftime('%Y%m%d')
     calendar_df = get_calendar_df()
     factor_data_df = get_factor_data_df()
-    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='P11701', fund_type='private', start_date='20171201', end_date='20211231', frequency='day', backup_period='近六月', convertible=True, equity=True).get_all()
-    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='P11701', fund_type='private', start_date='20171201', end_date='20211231', frequency='season').get_all()
+    fund_nav_series = get_fund_nav_series(fund_type, fund_id, start_date_backup, end_date)
+    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_nav_series, fund_id=fund_id, fund_type=fund_type, start_date=start_date, end_date=end_date, frequency='day', backup_period='近六月', convertible=True, equity=True).get_all()
+    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_nav_series, fund_id=fund_id, fund_type=fund_type, start_date=start_date, end_date=end_date, frequency='season').get_all()
     print(res)
 
     # # 跑批
