@@ -362,11 +362,11 @@ class StyleAttribution:
 
 if __name__ == '__main__':
     # 公募样例：110011
-    # res = StyleAttribution(fund_id='002943', fund_type='mutual', start_date='20161220', end_date='20220105',
-    #                        factor_type='style_allo', benchmark_id='000300', nav_frequency='day').get_all(processed=True)
+    res = StyleAttribution(fund_id='000311', fund_type='mutual', start_date='20210906', end_date='20230307',
+                           factor_type='style_allo', benchmark_id='000985', nav_frequency='day').get_all(processed=True)
     # 私募样例
-    res = StyleAttribution(fund_id='6E0108', fund_type='private', start_date='20181211', end_date='20220915',
-                           factor_type='style_allo', benchmark_id='000985', nav_frequency='week').get_all()
+    # res = StyleAttribution(fund_id='P48477', fund_type='private', start_date='20181211', end_date='20220915',
+    #                        factor_type='style_allo', benchmark_id='000985', nav_frequency='week').get_all()
     # res = StyleAttribution(fund_id='SX8958', fund_type='private', start_date='20180112', end_date='20211021',
     #                        factor_type='style_allo', benchmark_id='000985', nav_frequency='week').get_all()
     # res = StyleAttribution(fund_id='SR4480', fund_type='private', start_date='20191220', end_date='20210702',
