@@ -447,4 +447,10 @@ class FEDB:
     def read_style_index(self):
         sql = "SELECT * from style_index_history"
         df = self.get_df(sql)
+        return df
+
+    def read_overseas_nav_given_codes(self, codes):
+        codes = '"' + '","'.join(codes) + '"'
+        sql = "select TICKER_SYMBOL, TRADE_DATE, NAV from fe_temp_data.overseas_nav where TICKER_SYMBOL in ({0})".format(codes)
+        df = self.get_df(sql)
         return df
\ No newline at end of file
