@@ -660,6 +660,12 @@ class HBDB:
         df = self.get_df(sql, db='funduser', page_size=200000)
         return df
 
+    def read_overseas_nav_given_codes(self, codes):
+        codes = "'" + "','".join(codes) + "'"
+        sql = "SELECT jjdm, jzrq, hbdr, hbcl, hbfh, fqdwjz FROM st_hedge.t_st_rhb WHERE jjdm in ({0}) AND m_opt_type <> '03'".format(codes)
+        df = self.get_df(sql, db='highuser', page_size=200000)
+        return df
+
 if __name__ == '__main__':
     dates = HBDB().read_fund_barra_attribution_given_code('450004')
     barra_style_exposure = HBDB().read_barra_style_exposure_given_dates(fund_zc_wmv['RECENT_TRADE_DATE'].unique().tolist())
