@@ -2,7 +2,7 @@ from hbshare.quant.CChen.cta_factor.factor_func import (
     factor_table_gen,
     load_factor,
     get_last_position,
-    factor_stats,
+    factor_summary,
     factor_plot,
     factor_compute
 )
