@@ -84,6 +84,7 @@ def pre_data(price, **kwargs):
     return data, liq_days, calendar, price_liq
 
 
+# 动量值计算
 def mom(data_t, calendar, i, ts_n, factor, data):
     lookback_date = calendar[i - ts_n]
 
@@ -98,6 +99,7 @@ def mom(data_t, calendar, i, ts_n, factor, data):
     return data_t
 
 
+# 基差动量值计算
 def basismom(data, data_c, ts_n):
     # matrix
     data = data[data['TDATE'] >= data_c['TDATE'].sort_values().tolist()[0]].reset_index(drop=True)
@@ -218,6 +220,7 @@ def xs_pos(data, direction=1, quantile=50):
     return xs_weight(data)
 
 
+# 时序动量
 def tsmom(start_date, name='mom', direction=1, price='CLOSE', factor='CLOSE', loop=False, **kwargs):
     data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
 
@@ -265,6 +268,7 @@ def tsmom(start_date, name='mom', direction=1, price='CLOSE', factor='CLOSE', lo
     return pos_summary(positions=positions)
 
 
+# 截面动量
 def xsmom(start_date, name='mom', direction=1, price='CLOSE', factor='CLOSE', loop=False, **kwargs):
     data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
 
@@ -313,6 +317,146 @@ def xsmom(start_date, name='mom', direction=1, price='CLOSE', factor='CLOSE', lo
     return pos_summary(positions=positions)
 
 
+# 截面振幅
+def xstr(start_date, name='tr', direction=1, price='CLOSE', **kwargs):
+    data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
+
+    ts_n = kwargs['ts_n']
+    quantile = kwargs['quantile']
+
+    index_name = 'xs' + name + '_d' + str(ts_n) + '_q' + str(quantile)
+    start_index = get_index(calendar=calendar, date=start_date)
+
+    # matrix
+    mat_close = data.pivot(index='TDATE', columns='UCODE', values='CLOSE')
+    mat_close1 = mat_close.shift(1)
+    mat_high = data.pivot(index='TDATE', columns='UCODE', values='OPEN')
+    mat_low = data.pivot(index='TDATE', columns='UCODE', values='LOW')
+    mat_high[mat_high < mat_close1] = mat_close1[mat_high < mat_close1]
+    mat_low[mat_low > mat_close1] = mat_close1[mat_low > mat_close1]
+    factor_raw = (mat_high - mat_low) / mat_close1
+    factor = factor_raw.rolling(ts_n).mean()
+    factor = factor[~price_liq.isna()].replace([np.inf, -np.inf], np.nan)
+    # factor = (price_liq / price_liq.shift(ts_n) - 1)
+    pos, weights = xs_pos_mat(factor=factor, xs_q=quantile, direction=direction)
+    data_m = data.pivot(index='TDATE', columns='UCODE', values=price).shift(-1 if price in ['OPEN', 'SETTLE'] else 0)
+    positions = to_pos(
+        data=data_m,
+        factor=factor,
+        pos=pos,
+        weights=weights,
+        price_liq=price_liq,
+        index_name=index_name,
+        start_date=calendar[start_index]
+    )
+    return pos_summary(positions=positions)
+
+
+# 截面振幅波动率
+def xstrsigma(start_date, name='trsigma', direction=1, price='CLOSE', **kwargs):
+    data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
+
+    ts_n = kwargs['ts_n']
+    quantile = kwargs['quantile']
+
+    index_name = 'xs' + name + '_d' + str(ts_n) + '_q' + str(quantile)
+    start_index = get_index(calendar=calendar, date=start_date)
+
+    # matrix
+    mat_close = data.pivot(index='TDATE', columns='UCODE', values='CLOSE')
+    mat_close1 = mat_close.shift(1)
+    mat_high = data.pivot(index='TDATE', columns='UCODE', values='OPEN')
+    mat_low = data.pivot(index='TDATE', columns='UCODE', values='LOW')
+    mat_high[mat_high < mat_close1] = mat_close1[mat_high < mat_close1]
+    mat_low[mat_low > mat_close1] = mat_close1[mat_low > mat_close1]
+    factor_raw = (mat_high - mat_low) / mat_close1
+    factor = factor_raw.rolling(ts_n).std(ddof=1)
+    factor = factor[~price_liq.isna()].replace([np.inf, -np.inf], np.nan)
+    # factor = (price_liq / price_liq.shift(ts_n) - 1)
+    pos, weights = xs_pos_mat(factor=factor, xs_q=quantile, direction=direction)
+    data_m = data.pivot(index='TDATE', columns='UCODE', values=price).shift(-1 if price in ['OPEN', 'SETTLE'] else 0)
+    positions = to_pos(
+        data=data_m,
+        factor=factor,
+        pos=pos,
+        weights=weights,
+        price_liq=price_liq,
+        index_name=index_name,
+        start_date=calendar[start_index]
+    )
+    return pos_summary(positions=positions)
+
+
+# 截面振幅变化率
+def xstrchg(start_date, name='trchg', direction=1, price='CLOSE', **kwargs):
+    data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
+
+    ts_n = kwargs['ts_n']
+    quantile = kwargs['quantile']
+
+    index_name = 'xs' + name + '_d' + str(ts_n) + '_q' + str(quantile)
+    start_index = get_index(calendar=calendar, date=start_date)
+
+    # matrix
+    mat_close = data.pivot(index='TDATE', columns='UCODE', values='CLOSE')
+    mat_close1 = mat_close.shift(1)
+    mat_high = data.pivot(index='TDATE', columns='UCODE', values='OPEN')
+    mat_low = data.pivot(index='TDATE', columns='UCODE', values='LOW')
+    mat_high[mat_high < mat_close1] = mat_close1[mat_high < mat_close1]
+    mat_low[mat_low > mat_close1] = mat_close1[mat_low > mat_close1]
+    factor_raw = (mat_high - mat_low) / mat_close1
+    factor = factor_raw - factor_raw.shift(ts_n) - 1
+    factor = factor[~price_liq.isna()].replace([np.inf, -np.inf], np.nan)
+    # factor = (price_liq / price_liq.shift(ts_n) - 1)
+    pos, weights = xs_pos_mat(factor=factor, xs_q=quantile, direction=direction)
+    data_m = data.pivot(index='TDATE', columns='UCODE', values=price).shift(-1 if price in ['OPEN', 'SETTLE'] else 0)
+    positions = to_pos(
+        data=data_m,
+        factor=factor,
+        pos=pos,
+        weights=weights,
+        price_liq=price_liq,
+        index_name=index_name,
+        start_date=calendar[start_index]
+    )
+    return pos_summary(positions=positions)
+
+
+# 截面收盘价分位数（位于振幅）
+def xscloseintr(start_date, name='closeintr', direction=1, price='CLOSE', **kwargs):
+    data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
+
+    ts_n = kwargs['ts_n']
+    quantile = kwargs['quantile']
+
+    index_name = 'xs' + name + '_d' + str(ts_n) + '_q' + str(quantile)
+    start_index = get_index(calendar=calendar, date=start_date)
+
+    # matrix
+    mat_close = data.pivot(index='TDATE', columns='UCODE', values='CLOSE')
+    mat_close1 = mat_close.shift(1)
+    mat_high = data.pivot(index='TDATE', columns='UCODE', values='OPEN')
+    mat_low = data.pivot(index='TDATE', columns='UCODE', values='LOW')
+    mat_high[mat_high < mat_close1] = mat_close1[mat_high < mat_close1]
+    mat_low[mat_low > mat_close1] = mat_close1[mat_low > mat_close1]
+    factor_raw = (mat_close - mat_low) / (mat_high - mat_low)
+    factor = factor_raw.rolling(ts_n).mean()
+    factor = factor[~price_liq.isna()].replace([np.inf, -np.inf], np.nan)
+    # factor = (price_liq / price_liq.shift(ts_n) - 1)
+    pos, weights = xs_pos_mat(factor=factor, xs_q=quantile, direction=direction)
+    data_m = data.pivot(index='TDATE', columns='UCODE', values=price).shift(-1 if price in ['OPEN', 'SETTLE'] else 0)
+    positions = to_pos(
+        data=data_m,
+        factor=factor,
+        pos=pos,
+        weights=weights,
+        price_liq=price_liq,
+        index_name=index_name,
+        start_date=calendar[start_index]
+    )
+    return pos_summary(positions=positions)
+
+
 def tsrev(start_date, loop=False, **kwargs):
     return tsmom(start_date=start_date, name='rev', direction=-1, loop=loop, **kwargs)
 
@@ -1199,19 +1343,27 @@ def xspoichg(start_date, price='CLOSE', loop=False, **kwargs):
     return xsmom(start_date=start_date, name='poichg', factor='POI', price=price, loop=loop, **kwargs)
 
 
-def xssigma(start_date, price='CLOSE', loop=False, **kwargs):
+def xspoisigma(start_date, price='CLOSE', loop=False, **kwargs):
+    return xssigma(start_date=start_date, name='poisigma', factor='POI', price=price, loop=loop, **kwargs)
+
+
+def xspvolsigma(start_date, price='CLOSE', loop=False, **kwargs):
+    return xssigma(start_date=start_date, name='pvolsigma', factor='PVOL', price=price, loop=loop, **kwargs)
+
+
+def xssigma(start_date, name='sigma', factor='CLOSE', price='CLOSE', loop=False, **kwargs):
     data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
 
     ts_n = kwargs['ts_n']
     quantile = kwargs['quantile']
 
-    index_name = 'xssigma_d' + str(ts_n) + '_q' + str(quantile)
+    index_name = 'xs' + name + '_d' + str(ts_n) + '_q' + str(quantile)
     start_index = get_index(calendar=calendar, date=start_date)
 
     if loop:
 
         positions = pd.DataFrame()
-        data_close = data.pivot(index='TDATE', columns='UCODE', values='CLOSE')
+        data_close = data.pivot(index='TDATE', columns='UCODE', values=factor)
         data_vol = data.pivot(index='TDATE', columns='UCODE', values='VOL')
         data_sigma = data_close.pct_change().rolling(ts_n).std(ddof=1) * np.sqrt(240)
 
@@ -1221,7 +1373,7 @@ def xssigma(start_date, price='CLOSE', loop=False, **kwargs):
             ucode_vol = data_vol.columns[data_vol.loc[t_date, :] > kwargs['min_volume']].tolist()
             ucode_liq = data_vol.columns[
                 data_vol.loc[calendar[i - max(liq_days, ts_n)], :] > kwargs['min_volume']
-            ].tolist()
+                ].tolist()
             ucode = list(set(ucode_liq).intersection(set(ucode_vol)))
             data_t = pre_loop(
                 i=i,
@@ -1242,7 +1394,7 @@ def xssigma(start_date, price='CLOSE', loop=False, **kwargs):
             print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
     else:
         # matrix
-        factor_raw = data.pivot(index='TDATE', columns='UCODE', values='CLOSE').pct_change()
+        factor_raw = data.pivot(index='TDATE', columns='UCODE', values=factor).pct_change()
         factor = factor_raw.rolling(ts_n).std(ddof=1)
         factor = factor[~price_liq.isna()].replace([np.inf, -np.inf], np.nan)
         pos, weights = xs_pos_mat(
@@ -1263,6 +1415,38 @@ def xssigma(start_date, price='CLOSE', loop=False, **kwargs):
     return pos_summary(positions=positions)
 
 
+# Coefficient of Variation
+def xscv(start_date, name='cv', factor='CLOSE', price='CLOSE', **kwargs):
+    data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
+
+    ts_n = kwargs['ts_n']
+    quantile = kwargs['quantile']
+
+    index_name = 'xs' + name + '_d' + str(ts_n) + '_q' + str(quantile)
+    start_index = get_index(calendar=calendar, date=start_date)
+
+    # matrix
+    factor_raw = data.pivot(index='TDATE', columns='UCODE', values=factor).pct_change()
+    factor = factor_raw.rolling(ts_n).std(ddof=1) / factor_raw.rolling(ts_n).mean().abs()
+    factor = factor[~price_liq.isna()].replace([np.inf, -np.inf], np.nan)
+    pos, weights = xs_pos_mat(
+        factor=factor,
+        xs_q=quantile,
+        # direction=-1
+    )
+    data_m = data.pivot(index='TDATE', columns='UCODE', values=price).shift(-1 if price in ['OPEN', 'SETTLE'] else 0)
+    positions = to_pos(
+        data=data_m,
+        factor=factor,
+        pos=pos,
+        weights=weights,
+        price_liq=price_liq,
+        index_name=index_name,
+        start_date=calendar[start_index]
+    )
+    return pos_summary(positions=positions)
+
+
 def xsskew(start_date, price='CLOSE', loop=False, **kwargs):
     data, liq_days, calendar, price_liq = pre_data(price, **kwargs)
 
