@@ -7,7 +7,8 @@ from hbshare.quant.CChen.cta_factor.const import stock_data_indicator
 from hbshare.quant.CChen.cta_factor.factor_algo import (
     tscarry, carry, tsmom, tsrev, xsmom, xsrev, tswr, xswr, mr, mrchg, tsmr,
     tsbasismom, xsbasismom, tspoichg, xspoichg, xssigma,
-    xsmr, xstsmr, xsmrchg, tspvolchg, xspvolchg, tsskew, xsskew, xsmr2, xsmrchg2
+    xsmr, xstsmr, xsmrchg, tspvolchg, xspvolchg, tsskew, xsskew, xsmr2, xsmrchg2,
+    xstr, xstrsigma, xstrchg, xscloseintr
     # tsstock, xsstock
 )
 
@@ -184,6 +185,46 @@ def run(sql_path, sql_info):
         to_sql_path=sql_path
     )
 
+    factor_compute(
+        window_days_list=window_days_list,
+        hedge_ratio_list=[50, 75],
+        data=data,
+        factor_func=xstr,
+        liq_days=liq_d,
+        sql_info=sql_info,
+        to_sql_path=sql_path
+    )
+
+    factor_compute(
+        window_days_list=[10, 20, 30, 50, 90, 150, 250],
+        hedge_ratio_list=[50, 75],
+        data=data,
+        factor_func=xstrsigma,
+        liq_days=liq_d,
+        sql_info=sql_info,
+        to_sql_path=sql_path
+    )
+
+    factor_compute(
+        window_days_list=window_days_list,
+        hedge_ratio_list=[50, 75],
+        data=data,
+        factor_func=xstrchg,
+        liq_days=liq_d,
+        sql_info=sql_info,
+        to_sql_path=sql_path
+    )
+
+    factor_compute(
+        window_days_list=window_days_list,
+        hedge_ratio_list=[50, 75],
+        data=data,
+        factor_func=xscloseintr,
+        liq_days=liq_d,
+        sql_info=sql_info,
+        to_sql_path=sql_path
+    )
+
     factor_compute(
         window_days_list=window_days_list,
         data=data,
