@@ -485,38 +485,38 @@ def fund_data_to_hb(path_local, path_hb, sql_hb, database, tables=None):
             db_hb.close()
             print('\tdelete: ' + str(len(id_hb_diff)))
 
-        if t == 'fund_list':
-            id_local_diff = set(id_in_local[key_name]).difference(set(id_in_hb[key_name]))
-            if len(id_local_diff) == 0:
-                print('\tnew data: 0')
-                continue
-
-            if len(id_local_diff) == 1:
-                data_list_new = pd.read_sql_query(
-                    'select * from ' + t + ' where `' + key_name + '`=\'' + list(id_local_diff)[0] + '\'', path_local
-                )
-            else:
-                data_list_new = pd.read_sql_query(
-                    'select * from ' + t + ' where `' + key_name + '` in ' + str(tuple(id_local_diff)), path_local
-                )
-
-            data_list_new.to_sql(t, path_hb, if_exists='append', index=False)
-            print('\tnew data: ' + str(len(data_list_new)))
+        # if t == 'fund_list':
+        id_local_diff = set(id_in_local[key_name]).difference(set(id_in_hb[key_name]))
+        if len(id_local_diff) == 0:
+            print('\tnew data: 0')
+            continue
 
+        if len(id_local_diff) == 1:
+            data_list_new = pd.read_sql_query(
+                'select * from ' + t + ' where `' + key_name + '`=\'' + list(id_local_diff)[0] + '\'', path_local
+            )
         else:
-            data_hb_last_one = pd.read_sql_query(
-                'select `' + key_name + '` from ' + t + ' order by `' + key_name + '` desc limit 1', path_hb
+            data_list_new = pd.read_sql_query(
+                'select * from ' + t + ' where `' + key_name + '` in ' + str(tuple(id_local_diff)), path_local
             )
-            if len(data_hb_last_one) > 0:
-                id_num = data_hb_last_one[key_name][0]
-            else:
-                id_num = -1
 
-            data_work_new = pd.read_sql_query(
-                'select * from ' + t + ' where `' + key_name + '`>' + str(id_num), path_local
-            )
-            data_work_new.to_sql(t, path_hb, if_exists='append', index=False)
-            print('\tnew data: ' + str(len(data_work_new)))
+        data_list_new.to_sql(t, path_hb, if_exists='append', index=False)
+        print('\tnew data: ' + str(len(data_list_new)))
+
+        # else:
+        #     data_hb_last_one = pd.read_sql_query(
+        #         'select `' + key_name + '` from ' + t + ' order by `' + key_name + '` desc limit 1', path_hb
+        #     )
+        #     if len(data_hb_last_one) > 0:
+        #         id_num = data_hb_last_one[key_name][0]
+        #     else:
+        #         id_num = -1
+        #
+        #     data_work_new = pd.read_sql_query(
+        #         'select * from ' + t + ' where `' + key_name + '`>' + str(id_num), path_local
+        #     )
+        #     data_work_new.to_sql(t, path_hb, if_exists='append', index=False)
+        #     print('\tnew data: ' + str(len(data_work_new)))
 
 
 def load_gzb(name, start_date, end_date, db_path, table='gzb', c_diff=True):
@@ -528,11 +528,20 @@ def load_gzb(name, start_date, end_date, db_path, table='gzb', c_diff=True):
         + ' and 市价 > 0 order by 日期, 科目名称',
         db_path
     )
+    data_aum = pd.read_sql_query(
+        'select 日期, 基金名称, 科目代码, 市值 from ' + table
+        + ' where 日期<=' + end_date.strftime('%Y%m%d')
+        + ' and 日期>=' + start_date.strftime('%Y%m%d')
+        + ' and 基金名称="' + name + '"'
+        + ' and 科目代码 like \'%资产净值%\' and 市值占净值=100 order by 日期, 科目名称',
+        db_path
+    )
     data_info = data[['科目代码', '科目名称']].drop_duplicates(subset=['科目名称'])
     if c_diff:
         data = data.groupby(['日期', '科目名称'], as_index=False).sum()
         data = data.merge(data_info, on='科目名称', how='left')
     c_ind = data['科目代码'].str.split('310').str[0] == ''
+    stock_index = data['科目代码'].str.split('1102').str[0] == ''
     data.loc[c_ind, '品种'] = data.loc[c_ind, '科目代码'].apply(lambda x: re.findall(r'[a-zA-Z]+', x[8:])[0].upper())
     data['基金名称'] = name
     sec_df = pd.DataFrame()
@@ -541,9 +550,10 @@ def load_gzb(name, start_date, end_date, db_path, table='gzb', c_diff=True):
         sec['板块'] = i
         sec_df = sec_df.append(sec)
     data = data.merge(sec_df, on='品种', how='left')
+    data.loc[stock_index, '板块'] = '个股'
     data.loc[data['板块'].isnull().values == True, '板块'] = data.loc[data['板块'].isnull().values == True, '科目名称']
     data['板块'] = data['板块'].apply(
         lambda x: x.replace('私募证券投资基金', '').replace('私募投资基金', '').replace('私募基金', '')
     )
-    return data
+    return data, data_aum
 
