@@ -1599,15 +1599,16 @@ def market_win(start_date, end_date, freq='D', table='index_win', sql_path=sql_w
 
 
 def gzb_ts_positions(data, width=800, height=600):
-    data['绝对占比'] = data['市值占净值'].abs()
-    data_value = data.groupby(['日期'], as_index=False)['绝对占比'].sum().rename(columns={'绝对占比': '总比'})
-    data_value_by_sector = data.groupby(['日期', '板块'], as_index=False)['绝对占比'].sum()
+    data_d = data[0]
+    data_d['绝对占比'] = data_d['市值占净值'].abs()
+    data_value = data_d.groupby(['日期'], as_index=False)['绝对占比'].sum().rename(columns={'绝对占比': '总比'})
+    data_value_by_sector = data_d.groupby(['日期', '板块'], as_index=False)['绝对占比'].sum()
     data_value_by_sector = data_value_by_sector.merge(data_value, on='日期', how='left')
     data_value_by_sector['相对占比'] = data_value_by_sector['绝对占比'] / data_value_by_sector['总比'] * 100
 
-    data_value_net_by_sector = data.groupby(['日期', '板块'], as_index=False)['市值占净值'].sum()
+    data_value_net_by_sector = data_d.groupby(['日期', '板块'], as_index=False)['市值占净值'].sum()
 
-    sectors = data['板块'].drop_duplicates().tolist()
+    sectors = data_d['板块'].drop_duplicates().tolist()
     web = Bar().add_xaxis(data_value['日期'].tolist())
     web2 = Bar().add_xaxis(data_value['日期'].tolist())
     web3 = Bar().add_xaxis(data_value['日期'].tolist())
@@ -1661,10 +1662,11 @@ def gzb_ts_positions(data, width=800, height=600):
         ),
     )
 
+    # return web2
     web2.set_series_opts(
         label_opts=opts.LabelOpts(is_show=False)
     ).set_global_opts(
-        title_opts=opts.TitleOpts(title=data['基金名称'][0] + " 持仓分布"),
+        title_opts=opts.TitleOpts(title=data_d['基金名称'][0] + " 持仓分布"),
         legend_opts=opts.LegendOpts(
             orient="vertical",
             pos_top="5%",
@@ -1684,6 +1686,35 @@ def gzb_ts_positions(data, width=800, height=600):
             opts.DataZoomOpts(xaxis_index=[0, 2], range_start=0, range_end=100, type_="inside")
         ],
     )
+    # return web2
+    web_aum = Bar().add_xaxis(
+        xaxis_data=data[1]['日期'].tolist()
+    ).add_yaxis(
+        series_name='产品规模',
+        y_axis=data[1]['市值'].div(10000).round().tolist(),
+        # symbol="emptyCircle",
+        # is_symbol_show=False,
+        label_opts=opts.LabelOpts(is_show=False),
+        # is_selected=all_selected,
+        # yaxis_index=1,
+        ).set_global_opts(
+        # title_opts=opts.TitleOpts(title='产品规模'),
+        legend_opts=opts.LegendOpts(
+            # orient="vertical",
+            # pos_top="5%",
+            # pos_left='2%',
+            # pos_right='2%',
+            # type_='scroll',
+            is_show=False
+        ),
+        yaxis_opts=opts.AxisOpts(
+            name='产品规模（万元）',
+            axislabel_opts=opts.LabelOpts(formatter="{value}"),
+            min_='dataMin',
+            # max_=100,
+        ),
+    )
+    # web2.overlap(web_aum)
 
     grid = (
         Grid(
@@ -1694,27 +1725,36 @@ def gzb_ts_positions(data, width=800, height=600):
         ).add(
             web2,
             grid_opts=opts.GridOpts(
-                pos_right="18%",
+                pos_right="25%",
                 pos_left="8%",
                 pos_top="10%",
-                pos_bottom="60%"
+                pos_bottom="70%"
             ),
             # is_control_axis_index=True
         ).add(
             web,
             grid_opts=opts.GridOpts(
-                pos_right="18%",
+                pos_right="25%",
                 pos_left="8%",
-                pos_top="50%",
-                pos_bottom="35%"
+                pos_top="38%",
+                pos_bottom="50%"
             ),
             # is_control_axis_index=True
         ).add(
             web3,
             grid_opts=opts.GridOpts(
-                pos_right="18%",
+                pos_right="25%",
                 pos_left="8%",
-                pos_top="75%",
+                pos_top="58%",
+                pos_bottom="30%"
+            ),
+            # is_control_axis_index=True
+        ).add(
+            web_aum,
+            grid_opts=opts.GridOpts(
+                pos_right="25%",
+                pos_left="8%",
+                pos_top="78%",
                 # pos_bottom=""
             ),
             # is_control_axis_index=True
@@ -1725,6 +1765,7 @@ def gzb_ts_positions(data, width=800, height=600):
 
 
 def gzb_xs_position(data, width=800, height=600):
+    data = data[0]
     date = data['日期'][len(data) - 1]
     name = data['基金名称'][0]
     data = data[data['日期'] == date].reset_index(drop=True)
