@@ -51,6 +51,54 @@ class HoldingAnalysor:
         holding_df = pd.read_sql(sql_script, engine)
         holding_df['trade_date'] = holding_df['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
 
+        # all at trade_date
+        # sql_script = "SELECT * FROM private_fund_holding where trade_date = {}".format(self.trade_date)
+        # engine = create_engine(engine_params)
+        # holding_df = pd.read_sql(sql_script, engine)
+        # holding_df['trade_date'] = holding_df['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
+        # include_list = [
+        #     "白鹭精选量化鲲鹏十号",
+        #     "伯兄熙宁",
+        #     "宽德九臻",
+        #     "千衍六和3号",
+        #     "信弘500指数增强1号",
+        #     "因诺聚配中证500指数增强",
+        #     "衍复臻选指数增强一号",
+        #     "星阔广厦1号中证500指数增强",
+        #     "顽岩中证500指数增强1号",
+        #     "乾象中证500指数增强1号",
+        #     "赫富500指数增强一号",
+        #     "概率500指增2号",
+        #     "星阔广川21号"
+        # ]
+        # holding_df = holding_df[holding_df['fund_name'].isin(include_list)]
+        #
+        # import os
+        # import numpy as np
+        # path = "D:\\MarketInfoSaver"
+        # listdir = os.listdir(path)
+        # listdir = [x for x in listdir if x.split('_')[-1].split('.')[0] == '20221230'][0]
+        # date_t_data = pd.read_csv(os.path.join(path, listdir))
+        # date_t_data['ticker'] = date_t_data['ticker'].apply(lambda x: str(x).zfill(6))
+        #
+        # holding_df = holding_df.merge(date_t_data[['ticker', 'marketValue']], on='ticker')
+        # holding_df['marketValue'] /= 1e+8
+        # holding_df['total_weight'] = holding_df.groupby('fund_name')['weight'].transform('sum')
+        # holding_df['weight'] = 100 * holding_df['weight'] / holding_df['total_weight']
+        # # log处理
+        # holding_df['marketValue'] = np.log(holding_df['marketValue'])
+        # cut_list = list(np.linspace(2, 10, num=17))
+        # weight_dis = []
+        # for i in range(1, len(cut_list)):
+        #     x, y = cut_list[i - 1], cut_list[i]
+        #     name = "{}-{}".format(x, y)
+        #     a = holding_df[
+        #         (holding_df['marketValue'] >= x) & (holding_df['marketValue'] < y)].groupby(
+        #         'fund_name')['weight'].sum().to_frame(name)
+        #     weight_dis.append(a)
+        #
+        # weight_dis = pd.concat(weight_dis, axis=1).fillna(0.)
+
         # 招商量化精选 special
         # sql_script = "SELECT jsrq as end_date, ggrq as report_date, zqdm as ticker, " \
         #              "zjbl, zgbl, zlbl FROM st_fund.t_st_gm_gpzh where " \
@@ -70,7 +118,6 @@ class HoldingAnalysor:
         # print(num)
 
         return holding_df.set_index('ticker')['weight'] / 100.
-        # return holding_df.groupby('ticker')['weight'].sum() / 100. / num
 
     def _load_benchmark_weight_series(self, date):
         sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and SecuCode = '{}'".format(
@@ -612,7 +659,7 @@ class HoldingCompare:
 
 
 if __name__ == '__main__':
-    tmp = HoldingAnalysor("顽岩中证500指数增强1号", "20221231", "000905").get_construct_result(isPlot=False)
+    tmp = HoldingAnalysor("信弘500指数增强1号", "20221231", "000905").get_construct_result(isPlot=False)
 
     # dis_df = HoldingDistribution('20210310', '20220602', '因诺聚配中证500指数增强').get_construct_result()
     # print(dis_df)
