@@ -390,16 +390,16 @@ class PortfolioRiskPredict:
 
 
 if __name__ == '__main__':
-    # save_mkt_data_to_csv('20221226', '20230203')
+    save_mkt_data_to_csv('20230117', '20230117')
     # cal_illiq_factor('20191120', '20221130')
     # call_ivff_factor('20191120', '20221130')
 
-    factor = pd.read_csv("D:\\研究基地\\G-专题报告\\【2022.12】alpha策略的风控管理\\特质波动率因子.csv", dtype={"trade_date": str})
-    factor.rename(columns={"index": "ticker"}, inplace=True)
-    factor['ticker'] = factor['ticker'].apply(lambda x: str(x).zfill(6))
-    p_factor = factor_preprocess(factor, 'ivr')
-    ic = cal_factor_ic(p_factor, 'ivr')
-    p_factor.to_csv('D:\\研究基地\\G-专题报告\\【2022.12】alpha策略的风控管理\\特质波动率因子_处理后.csv', index=False)
+    # factor = pd.read_csv("D:\\研究基地\\G-专题报告\\【2022.12】alpha策略的风控管理\\特质波动率因子.csv", dtype={"trade_date": str})
+    # factor.rename(columns={"index": "ticker"}, inplace=True)
+    # factor['ticker'] = factor['ticker'].apply(lambda x: str(x).zfill(6))
+    # p_factor = factor_preprocess(factor, 'ivr')
+    # ic = cal_factor_ic(p_factor, 'ivr')
+    # p_factor.to_csv('D:\\研究基地\\G-专题报告\\【2022.12】alpha策略的风控管理\\特质波动率因子_处理后.csv', index=False)
 
     # factor = pd.read_csv("D:\\研究基地\\G-专题报告\\alpha策略的风控管理\\非流动性因子.csv", dtype={"trade_date": str})
     # factor['ticker'] = factor['ticker'].apply(lambda x: str(x).zfill(6))
