@@ -365,6 +365,11 @@ class FundTracker:
         cate_ret = cate_ret.sort_index()
         cate_nav = (1 + cate_ret).cumprod()
 
+        dt_df = pd.DataFrame({"trade_date": cate_nav.index})
+        dt_df['month'] = dt_df['trade_date'].apply(lambda x: datetime.datetime.strptime(x, "%Y%m%d").month)
+        month_end = dt_df[dt_df['month'].shift(-1) != dt_df['month']]['trade_date'].tolist()
+        month_alpha = cate_nav.reindex(month_end).pct_change().dropna()
+
         nav_df = (1 + return_df).cumprod()
         nav_df.loc[self.start_date] = 1.
         nav_df = nav_df.sort_index()
@@ -414,8 +419,10 @@ class FundTracker:
         compare_df.loc[self.start_date] = 1.
         compare_df = compare_df.sort_index()
 
-        return portfolio_index_df, rel_strength, cate_nav, nav_df, compare_df
+        return month_alpha, portfolio_index_df, rel_strength, cate_nav, nav_df, compare_df
+
 
 if __name__ == '__main__':
     # write_config_to_DB("tracker", is_increment=0)
-    FundTracker('20201231', '20230210').alpha_analysis()
\ No newline at end of file
+    # FundTracker('20220826', '20230210').alpha_analysis()
+    FundTracker('20220701', '20230224').alpha_analysis()
\ No newline at end of file
